#!/usr/bin/env python3

from wheel import Wheel
from sensor import Sensor
from odometry import Odometry
from planet import Direction
from time import sleep


def capat100(value):
    if(value > 100):
        return 100
    elif(value < -100):
        return -100
    else:
        return value


class Move:
    def __init__(self):
        try:
            self._wheel_l = Wheel('outB')
            self._wheel_r = Wheel('outC')
            self._sensor = Sensor()
            self._calibrate()
        except OSError:
            print("ERROR: Cannot find Motor/Sensor")
        self.defaultspeed = 20
        self._odometry = Odometry()

    '''
    determine maximum and minimum brightness of lines/white space
    depending on environment lighting
    '''
    def _calibrate(self):
        self._sensor.calibrateRGB()
        self._sensor.calibrateBrightness()
        # pass

    '''
    Function to correct errors should the robot wander astray
    '''

    def _aligntoedge(self):
        self._wheel_l.speed_set(20)
        self._wheel_r.speed_set(-20)
        self._wheel_l.run()
        self._wheel_r.run()
        self._sensor.refresh()
        while(self._sensor.getbrightness() > self._sensor.edgebrightness):
            self._sensor.refresh()
        self._wheel_l.stop()
        self._wheel_r.stop()
        self._wheel_l.turnbyamount(20, 20)
        self._wheel_r.turnbyamount(-20, 20)

    '''
    uses odometry and the planet object to map edges that are connected to
    current node.

    Use either relative motor pos or timed driving to center on node.
    Next turn around and detect edges through drops in _sensor.getbrightness()
    '''
    def getstationedges(self):
        found_edges = []
        self._odometry.reset()
        while(True):
            self._aligntoedge()
            self._odometry.pos_update((self._wheel_l.getmovement(), self._wheel_r.getmovement()))
            if(self._odometry.angle_get() in found_edges):
                break
            else:
                found_edges.append(self._odometry.angle_get())
                self._wheel_l.turnbyamount(20, 20)
                self._wheel_r.turnbyamount(-20, 20)
                sleep(2)
        return found_edges


    def turnto(self, direction):
        turnval = self._odometry.dir2ticks(direction)
        self._wheel_l.turnbyamount(turnval, 90)
        self._wheel_r.turnbyamount(-turnval, 90)

    def traversetonextstation(self, isknownstation):
        self._wheel_l.speed_set(self._sensor.edgebrightness//2)
        self._wheel_r.speed_set(self._sensor.edgebrightness//2)
        self._wheel_l.run()
        self._wheel_r.run()
        while (not self._sensor.bumperwaspressed and self._sensor.lastcolor is None):
            self._sensor.refresh()
            self._sensor.checkbumper()
            self._wheel_l.speed_set(capat100(self._sensor.edgebrightness/2 - self._sensor.getbrightness()/2)+self.defaultspeed)
            self._wheel_r.speed_set(capat100(self._sensor.getbrightness()/2 - self._sensor.edgebrightness/2)+self.defaultspeed)
            self._sensor.getcolor()
            if(not isknownstation):
                self._odometry.pos_update((self._wheel_l.getmovement(), self._wheel_r.getmovement()))
        self._wheel_l.stop()
        self._wheel_r.stop()
